/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 * $Id: PathPanel.java,v 1.1 2000/05/30 07:31:16 pmahar Exp $
 */
package org.enhydra.kelp.common.wizard.importer;

// Kelp imports
import org.enhydra.kelp.common.ValidationException;
import org.enhydra.kelp.common.ValidationUtil;
import org.enhydra.kelp.common.swing.SwingUtil;

// Standard imports
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.*;
import java.io.File;

/**
 * Panel for entering the default options: project name, package
 * and destination.
 *
 * @author Paul Mahar
 * @version $Revision: 1.1 $
 */
public class PathPanel extends ImporterPanel {
    transient private JPanel               panelSource;
    transient private JTextField           textSource;
    transient private JButton              buttonSource;
    transient private JPanel               panelResource;
    transient private JTextField           textResource;
    transient private JButton              buttonResource;

    //
    transient private JPanel              panelFiller;
    transient private GridBagLayout       layoutMain;
    transient private GridBagLayout       layoutSource;
    transient private GridBagLayout       layoutResource;
    transient private LocalButtonListener buttonListener;
    transient private TitledBorder        borderSource;
    transient private TitledBorder        borderResource;


    /**
     * Create default option panel 1 for entering the
     * following default options: project name, package and
     * destination.
     */
    public PathPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Read project name, package and destination values from the
     * option set into the swing controls.
     *
     * @exception GeneratorException
     * Thrown if unable to update Swing controls with option set values.
     */
    public void readOptions() {
        File file = null;

        // source path
        file = getImporter().getSource();
        if (file == null) {
          textSource.setText(new String());
        } else {
          textSource.setText(file.getAbsolutePath());
        }

        // resource path
        file = getImporter().getResource();
        if (file == null) {
          textResource.setText(new String());
        } else {
          textResource.setText(file.getAbsolutePath());
        }
    }

    /**
     * Write project name, package and destination values from the
     * swing controls into the option set.
     *
     * @exception GeneratorException
     * Thrown if unable to update option set from Swing control values.
     */
    public void writeOptions() throws ValidationException {
        String value = new String();

        if (getImporter() == null) {
          throw new ValidationException("Importer not set: cannot write options");
        }
        // source
        value = textSource.getText().trim();
        getImporter().setSource(new File(value));

        // resource
        value = textResource.getText().trim();
        getImporter().setResource(new File(value));
    }

    /**
     * Validate project name, package and destination values
     * in the swing controls.
     *
     * @exception ValidationException
     * Thrown if swing control values are not valid for the
     * current option set.
     */
    public void validateOptions() throws ValidationException {
        String value = new String();

        // package
        value = textSource.getText().trim();
        if (!ValidationUtil.isDirectory(value)) {
            throw new ValidationException("Source directory is not valid. "
                                          + "Directory not found.");
        }

        // destination
        value = textResource.getText().trim();
        if (!ValidationUtil.isDirectory(value)) {
            throw new ValidationException("Resource directory is not valid. "
                                          + "Directory not found.");
        }
    }

    /**
     * Get the title to use on the current page.
     *
     * @return
     * A string to place at the top of a CodeGen wizard panel.
     */
    public String getPageTitle() {
        return "Path details";
    }

    /**
     * Get the instructions for entering option values for the
     * current page.
     *
     * @return
     * A string to place below the page title.
     */
    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append("The source path is the directory at the root of Java ");
        buf.append("package names. The source path is often the same as ");
        buf.append("the project root or a \"src\" subdirectory under the ");
        buf.append("project root.\n\n");
        buf.append("The resource directory is the relative root referenced ");
        buf.append("by web page links. The resource directory stores ");
        buf.append("image, html and wml file. It is often a directory named ");
        buf.append("\"resources\" that is parallel to the presentation ");
        buf.append("package directory.");
        return buf.toString();
    }

    public void activated() {
      initPaths();
    }

    private void initPaths() {
      File file = null;
      if (textSource.getText().trim().length() == 0) {
         file = getImporter().getDefaultSource();
         if (file != null) {
           try {
             getImporter().setSource(file);
           } catch (ValidationException e) {
             e.printStackTrace();
           }
           textSource.setText(file.getAbsolutePath());
         }
      }
      if (textResource.getText().trim().length() == 0) {
        file = getImporter().getDefaultResource();
        if (file != null) {
       	   textResource.setText(file.getAbsolutePath());
        }
      }
    }

    // /
    // /

    /**
     * Method declaration
     *
     *
     * @exception Exception
     *
     * @see
     */
    private void jbInit() throws Exception {
        textSource =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        panelSource = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        panelResource = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());

        layoutSource =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutResource =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());

        buttonSource = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());

        buttonResource = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());

        textResource = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        textSource.setText("SOURCE");
        textResource.setText("RESOURCE");
        buttonSource.setText("Set...");
        buttonResource.setText("Set...");

        panelSource.setLayout(layoutSource);
        panelSource.add(textSource,
                      new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(5, 5, 5, 5), 0, 0));
        panelSource.add(buttonSource,
                      new GridBagConstraints(1, 0, 1, 2, 0.0, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.NONE,
                                             new Insets(5, 5, 5, 20), 0, 0));


        panelResource.setLayout(layoutResource);
        panelResource.add(textResource,
                      new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(5, 5, 5, 5), 0, 0));
        panelResource.add(buttonResource,
                      new GridBagConstraints(1, 0, 1, 2, 0.0, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.NONE,
                                             new Insets(5, 5, 5, 20), 0, 0));
        this.setLayout(layoutMain);
        this.add(panelSource,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));

        this.add(panelResource,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void pmInit() {
        Border border = BorderFactory.createEtchedBorder();

        borderSource = BorderFactory.createTitledBorder(border);
        borderSource.setTitle("Source directory");

        borderResource = BorderFactory.createTitledBorder(border);
        borderResource.setTitle("Resource directory");

        panelSource.setBorder(borderSource);
        panelResource.setBorder(borderResource);

        buttonListener = new LocalButtonListener();
        buttonResource.addActionListener(buttonListener);
        buttonSource.addActionListener(buttonListener);
        textResource.setText(new String());
        textSource.setText(new String());
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void browseForFile() {
        File destDir = null;

        destDir =
            SwingUtil.getDirectoryChoice(this, textResource.getText(),
                                         "Choose destination directory");
        if (destDir != null) {
            textResource.setText(destDir.getAbsolutePath());
        }
    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     * @version $Revision: 1.1 $
     */
    private class LocalButtonListener implements ActionListener {

        /**
         * Method declaration
         *
         *
         * @param event
         *
         * @see
         */
        public void actionPerformed(ActionEvent event) {
            browseForFile();
        }

    }
}
